///
/// @file        clk_imx6.h
/// @copyright   Copyright (c) 2014 Toradex AG
/// $Author: raja.sivasankaran $
/// $Rev: 3075 $ 
/// $Date: 2016-02-01 11:04:17 +0530 (Mon, 01 Feb 2016) $
/// @brief       Library to manage clock settings
/// @target      Colibri iMX6S, iMX6DL, Apalis iMX6Q, iMX6D

#ifndef _CLK_IMX6_H_
#define _CLK_IMX6_H_

#include "TdxCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

/// Imx6 Clock ID
typedef enum
{
    DDK_CLOCK_GATE_INDEX_AIPS_TZ1                           = 0,    // CGR0[1:0]
    DDK_CLOCK_GATE_INDEX_AIPS_TZ2                           = 1,    // CGR0[3:2]
    DDK_CLOCK_GATE_INDEX_APBHDMA_HCLK                       = 2,    // CGR0[5:4]
    DDK_CLOCK_GATE_INDEX_ASRC_CLK                           = 3,    // CGR0[7:6]
    DDK_CLOCK_GATE_INDEX_CAAM_SECURE_MEM                    = 4,    // CGR0[9:8]
    DDK_CLOCK_GATE_INDEX_CAAM_WRAPPER_ACLK                  = 5,    // CGR0[11:10]
    DDK_CLOCK_GATE_INDEX_CAAM_WRAPPER_IPG                   = 6,    // CGR0[13:12]
    DDK_CLOCK_GATE_INDEX_CAN1                               = 7,    // CGR0[15:14]
    DDK_CLOCK_GATE_INDEX_CAN1_SERIAL                        = 8,    // CGR0[17:16]
    DDK_CLOCK_GATE_INDEX_CAN2                               = 9,    // CGR0[19:18]
    DDK_CLOCK_GATE_INDEX_CAN2_SERIAL                        = 10,   // CGR0[21:20]
    DDK_CLOCK_GATE_INDEX_CHEETAH_DBG                        = 11,   // CGR0[23:22]
    DDK_CLOCK_GATE_INDEX_DCIC1                              = 12,   // CGR0[25:24]
    DDK_CLOCK_GATE_INDEX_DCIC2                              = 13,   // CGR0[27:26]
    DDK_CLOCK_GATE_INDEX_DTCP_DTCP                          = 14,   // CGR0[29:28]
    DDK_CLOCK_GATE_INDEX_R1                                 = 15,   // CGR0[31:30]

    DDK_CLOCK_GATE_INDEX_ECSPI1                             = 16,   // CGR1[1:0]
    DDK_CLOCK_GATE_INDEX_ECSPI2                             = 17,   // CGR1[3:2]
    DDK_CLOCK_GATE_INDEX_ECSPI3                             = 18,   // CGR1[5:4]
    DDK_CLOCK_GATE_INDEX_ECSPI4                             = 19,   // CGR1[7:6]
    DDK_CLOCK_GATE_INDEX_ECSPI5_I2C4                        = 20,   // CGR1[9:8]        // it's ECSPI5 on Q/D and I2C4 on S/DL
    DDK_CLOCK_GATE_INDEX_ENET                               = 21,   // CGR1[11:10]
    DDK_CLOCK_GATE_INDEX_EPIT1                              = 22,   // CGR1[13:12]
    DDK_CLOCK_GATE_INDEX_EPIT2                              = 23,   // CGR1[15:14]
    DDK_CLOCK_GATE_INDEX_ESAI                               = 24,   // CGR1[17:16]
    DDK_CLOCK_GATE_INDEX_R2                                 = 25,   // CGR1[19:18]
    DDK_CLOCK_GATE_INDEX_GPT                                = 26,   // CGR1[21:20]
    DDK_CLOCK_GATE_INDEX_GPT_SERIAL                         = 27,   // CGR1[23:22]
    DDK_CLOCK_GATE_INDEX_GPU2D                              = 28,   // CGR1[25:24]
    DDK_CLOCK_GATE_INDEX_GPU3D                              = 29,   // CGR1[27:26]
    DDK_CLOCK_GATE_INDEX_R3                                 = 30,   // CGR1[29:28]
    DDK_CLOCK_GATE_INDEX_R4                                 = 31,   // CGR1[31:30]

    DDK_CLOCK_GATE_INDEX_HDMI_TX_IAHBCLK                    = 32,   // CGR2[1:0]
    DDK_CLOCK_GATE_INDEX_R5                                 = 33,   // CGR2[3:2]
    DDK_CLOCK_GATE_INDEX_HDMI_TX_ISFR                       = 34,   // CGR2[5:4]
    DDK_CLOCK_GATE_INDEX_I2C1_SERIAL                        = 35,   // CGR2[7:6]
    DDK_CLOCK_GATE_INDEX_I2C2_SERIAL                        = 36,   // CGR2[9:8]
    DDK_CLOCK_GATE_INDEX_I2C3_SERIAL                        = 37,   // CGR2[11:10]
    DDK_CLOCK_GATE_INDEX_IIM                                = 38,   // CGR2[13:12]
    DDK_CLOCK_GATE_INDEX_IOMUX_IPT                          = 39,   // CGR2[15:14]
    DDK_CLOCK_GATE_INDEX_IPMUX1                             = 40,   // CGR2[17:16]
    DDK_CLOCK_GATE_INDEX_IPMUX2                             = 41,   // CGR2[19:18]
    DDK_CLOCK_GATE_INDEX_IPMUX3                             = 42,   // CGR2[21:20]
    DDK_CLOCK_GATE_INDEX_IPSYNC_IP2APB_TZASC2_IPG_MASTER    = 43,   // CGR2[23:22]
    DDK_CLOCK_GATE_INDEX_IPSYNC_IP2APB_TZASC2_IPG           = 44,   // CGR2[25:24]
    DDK_CLOCK_GATE_INDEX_IPSYNC_VDOA_IPG_MASTER             = 45,   // CGR2[27:26]
    DDK_CLOCK_GATE_INDEX_R6                                 = 46,   // CGR2[29:28]
    DDK_CLOCK_GATE_INDEX_R7                                 = 47,   // CGR2[31:30]

    DDK_CLOCK_GATE_INDEX_IPU1_IPU                           = 48,   // CGR3[1:0]
    DDK_CLOCK_GATE_INDEX_IPU1_IPU_DI0                       = 49,   // CGR3[3:2]
    DDK_CLOCK_GATE_INDEX_IPU1_IPU_DI1                       = 50,   // CGR3[5:4]
    DDK_CLOCK_GATE_INDEX_IPU2_IPU                           = 51,   // CGR3[7:6]
    DDK_CLOCK_GATE_INDEX_IPU2_IPU_DI0                       = 52,   // CGR3[9:8]
    DDK_CLOCK_GATE_INDEX_IPU2_IPU_DI1                       = 53,   // CGR3[11:10]
    DDK_CLOCK_GATE_INDEX_LDB_DI0                            = 54,   // CGR3[13:12]
    DDK_CLOCK_GATE_INDEX_LDB_DI1                            = 55,   // CGR3[15:14]
    DDK_CLOCK_GATE_INDEX_MIPI_CORE_CFG                      = 56,   // CGR3[17:16]
    DDK_CLOCK_GATE_INDEX_MLB                                = 57,   // CGR3[19:18]
    DDK_CLOCK_GATE_INDEX_MMDC_CORE_ACLK_FAST_CORE_P0        = 58,   // CGR3[21:20]
    DDK_CLOCK_GATE_INDEX_MMDC_CORE_ACLK_FAST_CORE_P1        = 59,   // CGR3[23:22]
    DDK_CLOCK_GATE_INDEX_MMDC_CORE_IPG_P0                   = 60,   // CGR3[25:24]
    DDK_CLOCK_GATE_INDEX_MMDC_CORE_IPG_P1                   = 61,   // CGR3[27:26]
    DDK_CLOCK_GATE_INDEX_OCRAM                              = 62,   // CGR3[29:28]
    DDK_CLOCK_GATE_INDEX_OPENVGAXI_ROOT                     = 63,   // CGR3[31:30]

    DDK_CLOCK_GATE_INDEX_PCIE_ROOT                          = 64,   // CGR4[1:0]
    DDK_CLOCK_GATE_INDEX_PERFMON1_APB                       = 65,   // CGR4[3:2]
    DDK_CLOCK_GATE_INDEX_PERFMON2_APB                       = 66,   // CGR4[4:3]
    DDK_CLOCK_GATE_INDEX_PERFMON3_APB                       = 67,   // CGR4[7:6]
    DDK_CLOCK_GATE_INDEX_PL301_FAST1_S133                   = 68,   // CGR4[9:8]
    DDK_CLOCK_GATE_INDEX_R8                                 = 69,   // CGR4[11:10]
    DDK_CLOCK_GATE_INDEX_PL301_PER1_BCH                     = 70,   // CGR4[13:12]
    DDK_CLOCK_GATE_INDEX_PL301_PER2_MAIN                    = 71,   // CGR4[15:14]
    DDK_CLOCK_GATE_INDEX_PWM1                               = 72,   // CGR4[17:16]
    DDK_CLOCK_GATE_INDEX_PWM2                               = 73,   // CGR4[19:18]
    DDK_CLOCK_GATE_INDEX_PWM3                               = 74,   // CGR4[21:20]
    DDK_CLOCK_GATE_INDEX_PWM4                               = 75,   // CGR4[23:22]
    DDK_CLOCK_GATE_INDEX_RAWNAND_U_BCH_INPUT_APB            = 76,   // CGR4[25:24]
    DDK_CLOCK_GATE_INDEX_RAWNAND_U_GPMI_BCH_INPUT_BCH       = 77,   // CGR4[27:26]
    DDK_CLOCK_GATE_INDEX_RAWNAND_U_GPMI_BCH_INPUT_GPMI_IO   = 78,   // CGR4[29:28]
    DDK_CLOCK_GATE_INDEX_RAWNAND_U_GPMI_INPUT_APB           = 79,   // CGR4[31:30]

    DDK_CLOCK_GATE_INDEX_ROM                                = 80,   // CGR5[1:0]
    DDK_CLOCK_GATE_INDEX_R9                                 = 81,   // CGR5[3:2]
    DDK_CLOCK_GATE_INDEX_SATA                               = 82,   // CGR5[5:4]
    DDK_CLOCK_GATE_INDEX_SDMA                               = 83,   // CGR5[7:6]
    DDK_CLOCK_GATE_INDEX_R10                                = 84,   // CGR5[9:8]
    DDK_CLOCK_GATE_INDEX_R11                                = 85,   // CGR5[11:10]
    DDK_CLOCK_GATE_INDEX_SPBA                               = 86,   // CGR5[13:12]
    DDK_CLOCK_GATE_INDEX_SPDIF                              = 87,   // CGR5[15:14]
    DDK_CLOCK_GATE_INDEX_R12                                = 88,   // CGR5[17:16]
    DDK_CLOCK_GATE_INDEX_SSI1                               = 89,   // CGR5[19:18]
    DDK_CLOCK_GATE_INDEX_SSI2                               = 90,   // CGR5[21:20]
    DDK_CLOCK_GATE_INDEX_SSI3                               = 91,   // CGR5[23:22]
    DDK_CLOCK_GATE_INDEX_UART                               = 92,   // CGR5[25:24]
    DDK_CLOCK_GATE_INDEX_UART_SERIAL                        = 93,   // CGR5[27:26]
    DDK_CLOCK_GATE_INDEX_R13                                = 94,   // CGR5[29:28]
    DDK_CLOCK_GATE_INDEX_R14                                = 95,   // CGR5[31:30]

    DDK_CLOCK_GATE_INDEX_USBOH3                             = 96,   // CGR6[1:0]
    DDK_CLOCK_GATE_INDEX_USDHC1                             = 97,   // CGR6[3:2]
    DDK_CLOCK_GATE_INDEX_USDHC2                             = 98,   // CGR6[5:4]
    DDK_CLOCK_GATE_INDEX_USDHC3                             = 99,   // CGR6[7:6]
    DDK_CLOCK_GATE_INDEX_USDHC4                             = 100,  // CGR6[9:8]
    DDK_CLOCK_GATE_INDEX_EMI_SLOW                           = 101,  // CGR6[11:10]
    DDK_CLOCK_GATE_INDEX_VDOAXI                             = 102,  // CGR6[13:12]
    DDK_CLOCK_GATE_INDEX_VPU                                = 103,  // CGR6[15:14]
    DDK_CLOCK_GATE_INDEX_R15                                = 104,  // CGR6[17:16]
    DDK_CLOCK_GATE_INDEX_R16                                = 105,  // CGR6[19:18]
    DDK_CLOCK_GATE_INDEX_R17                                = 106,  // CGR6[21:20]
    DDK_CLOCK_GATE_INDEX_R18                                = 107,  // CGR6[23:22]
    DDK_CLOCK_GATE_INDEX_R19                                = 108,  // CGR6[25:24]
    DDK_CLOCK_GATE_INDEX_R20                                = 109,  // CGR6[27:26]
    DDK_CLOCK_GATE_INDEX_R21                                = 110,  // CGR6[29:28]
    DDK_CLOCK_GATE_INDEX_R22                                = 111,  // CGR6[31:30]

    DDK_CLOCK_GATE_INDEX_R23                                = 112,  // CGR7[1:0]
    DDK_CLOCK_GATE_INDEX_R24                                = 113,  // CGR7[3:2]
    DDK_CLOCK_GATE_INDEX_R25                                = 114,  // CGR7[5:4]
    DDK_CLOCK_GATE_INDEX_R26                                = 115,  // CGR7[7:6]
    DDK_CLOCK_GATE_INDEX_R27                                = 116,  // CGR7[9:8]
    DDK_CLOCK_GATE_INDEX_R28                                = 117,  // CGR7[11:10]
    DDK_CLOCK_GATE_INDEX_R29                                = 118,  // CGR7[13:12]
    DDK_CLOCK_GATE_INDEX_R30                                = 119,  // CGR7[15:14]
    DDK_CLOCK_GATE_INDEX_R31                                = 120,  // CGR7[17:16]
    DDK_CLOCK_GATE_INDEX_R32                                = 121,  // CGR7[19:18]
    DDK_CLOCK_GATE_INDEX_R33                                = 122,  // CGR7[21:20]
    DDK_CLOCK_GATE_INDEX_R34                                = 123,  // CGR7[23:22]
    DDK_CLOCK_GATE_INDEX_R35                                = 124,  // CGR7[25:24]
    DDK_CLOCK_GATE_INDEX_R36                                = 125,  // CGR7[27:26]
    DDK_CLOCK_GATE_INDEX_R37                                = 126,  // CGR7[29:28]
    DDK_CLOCK_GATE_INDEX_R38                                = 127,  // CGR7[31:30]

    DDK_CLOCK_GATE_INDEX_ENUM_END                           = 128
} tImx6ClkId;

//******************************************************************************
/// Initialize HANDLE for the imx6 clock lib.\n
/// Allocate memory map and update to the handle.\n
/// This function does not modify any hardware registers.\n
/// @param[in]  registryPath  Registry path string value.e.g "SOFTWARE\\Toradex\\spi" , "SOFTWARE\\Toradex\\app"\n
///             Reserved for future use. Currently we don't need it.\n
///             NULL = do not use registry.
/// @return     Handle to the Clock Library. NULL if the function failed.
HANDLE Imx6Clk_Init(const TCHAR *registryPath);

//******************************************************************************
/// Deinit HANDLE for the imx6 clock lib.\n 
/// Free the allocated memory.
/// @param[in]    hClk            HANDLE of clock lib
/// @retval       TRUE            Success
/// @retval       FALSE           Failure
BOOL Imx6Clk_Deinit(HANDLE hClk);

//******************************************************************************
/// Enable / disable clock for a specific peripheral
/// @param[in]    hClk            HANDLE of clock lib
/// @param[in]    clockId         Clock ID (refer to enum tImx6ClkId)
/// @param[in]    enable          TRUE to enable clock \n FALSE to disable clock
/// @retval       TRUE            Success
/// @retval       FALSE           Failure
BOOL Imx6Clk_EnableClock(HANDLE hClk, tImx6ClkId clockId, BOOL enable);

//******************************************************************************
/// Request clock enable status for a specific peripheral.
/// @param[in]    hClk            HANDLE of clock lib 
/// @param[in]    clockId         Clock ID (refer to enum tImx6ClkId)
/// @retval       TRUE            The clock is currently enabled
/// @retval       FALSE           The clock is currently disabled
BOOL Imx6Clk_ClockEnabled(HANDLE hClk, tImx6ClkId clockId);

//******************************************************************************
/// Get the revision of this Clock lib
/// @param[out]     ver     library version, consist of major, minor and build version.
void Imx6Clk_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif